#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .ini import IniProviderBackend
import os


def get_provider(path):
    file_name, file_ext = os.path.splitext(path)
    return {
        '.ini': IniProviderBackend
    }.get(file_ext.lower())(path)

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import abc


class ConfigException(Exception):
    pass


class BaseProviderBackend(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self, *args, **kwargs):
        super(BaseProviderBackend, self).__init__(*args, **kwargs)

    @abc.abstractmethod
    def get(self, key, default_value):
        raise NotImplementedError

    @abc.abstractmethod
    def set(self, key, value):
        raise NotImplementedError

    @abc.abstractmethod
    def delete(self, key):
        raise NotImplementedError

    @abc.abstractmethod
    def to_dict(self, key):
        raise NotImplementedError


class FileProviderBackend(BaseProviderBackend):
    def __init__(self, file_path, *args, **kwargs):
        super(FileProviderBackend, self).__init__(*args, **kwargs)
        self._file_path = file_path

    @property
    def file_path(self):
        return self._file_path


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import eq_
from nose.tools import ok_
import os
from py_configurator import Locator
import unittest


class TestLocator(unittest.TestCase):

    def test_locator_construction(self):
        config_name = 'foo.bar'
        local_dir = './'
        system_dir = '/foo/bar/tmp'
        env_key = 'CONFIG_KEY'

        locator = Locator(
            env_key=env_key,
            config_name=config_name,
            local_dir=local_dir,
            system_dir=system_dir)

        eq_(locator.config_name, config_name)
        eq_(locator.local_dir, local_dir)
        eq_(locator.env_key, env_key)

        config_name_2 = 'foo.bar.2'
        local_dir_2 = '/foo/bar/tmp/2'
        env_key_2 = 'CONFIG_KEY_2'
        locator.config_name = config_name_2
        locator.local_dir = local_dir_2
        locator.env_key = env_key_2

        eq_(locator.config_name, config_name_2)
        eq_(locator.local_dir, local_dir_2)
        eq_(locator.env_key, env_key_2)

    def test_config_locator_get_config_search_paths(self):
        config_name = 'foo.bar'
        local_dir = './'
        system_dir = '/foo/bar/tmp'
        env_key = 'CONFIG_KEY'
        env_key_path = '/bar/config.path'

        os.environ[env_key] = env_key_path

        locator = Locator(
            env_key=env_key,
            config_name=config_name,
            local_dir=local_dir,
            system_dir=system_dir)

        config_search_paths = locator.get_config_paths()

        ok_(env_key_path in config_search_paths)
        ok_(os.path.join('./', config_name) in config_search_paths)
        ok_('/foo/bar/tmp/foo.bar' in config_search_paths)
        ok_(os.path.join(os.path.expanduser("~"), config_name) in
            config_search_paths)


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from mock import patch
from nose.tools import eq_
from py_configurator.config import Config
import unittest


class TestConfig(unittest.TestCase):

    @patch('py_configurator.backends.base.BaseProviderBackend')
    def test_config_get_with_mock_provider(self, mock_provider):
        mock_provider.get.return_value = "Foo"
        config = Config(mock_provider)
        eq_(config.get("bar"), "Foo")
        mock_provider.get.assert_called_once_with("bar", None)

    @patch('py_configurator.backends.base.BaseProviderBackend')
    def test_config_set_with_mock_provider(self, mock_provider):
        config = Config(mock_provider)
        config.set("boo", "moo")
        mock_provider.set.assert_called_once_with("boo", "moo")


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import ConfigException
from .base import FileProviderBackend
import yaml


class YamlProviderBackend(FileProviderBackend):
    def __init__(self, file_path, *args, **kwargs):
        super(YamlProviderBackend, self).__init__(file_path, *args, **kwargs)
        try:
            with open(file_path, 'r') as f:
                self._config = yaml.load(f)
        except Exception as e:
            raise ConfigException(e)

    def get(self, key, default_value=None):
        try:
            return self._get(self._config, key)
        except:
            return default_value

    def _get(self, target, key):
        if '.' not in key:
            return target[key]
        key, rest_of_key = key.split('.', 1)
        return self._get(target[key], rest_of_key)

    def set(self, key, value):
        try:
            return self._set(self._config, key, value)
        except Exception as e:
            raise ConfigException(e)

    def _set(self, target, key, value):
        if '.' not in key:
            target[key] = value
            return
        key, rest_of_key = key.split('.', 1)
        return self._set(target[key], rest_of_key, value)

    def delete(self, key):
        try:
            self._delete(self._config, key)
        except Exception as e:
            raise ConfigException(e)

    def _delete(self, target, key):
        if '.' not in key:
            del target[key]
            return
        key, rest_of_key = key.split('.', 1)
        return self._delete(target[key], rest_of_key)

    def to_dict(self):
        raise NotImplementedError

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Config(object):
    def __init__(self, provider, *args, **kwargs):
        super(Config, self).__init__(*args, **kwargs)
        self.provider = provider

    def get(self, name, default_value=None):
        return self.provider.get(name, default_value)

    def set(self, name, value):
        return self.provider.set(name, value)

    def delete(self, name):
        return self.provider.delete(name)

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from collections import namedtuple
version_info = namedtuple('version_info', ('major', 'minor', 'patch'))


VERSION = version_info(0, 0, 1)


__title__ = 'py_utilities'
__version__ = '{0.major}.{0.minor}.{0.patch}'.format(VERSION)
__author__ = 'Ryan Kanno'
__email__ = 'ryankanno@localkinegrinds.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 Ryan Kanno'


from .locator import Locator

__all__ = [Locator]

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import eq_
from nose.tools import raises
import os
from py_configurator.backends.base import ConfigException
from py_configurator.backends.ini import IniProviderBackend
import unittest


class TestIniProviderBackend(unittest.TestCase):

    def setUp(self):
        self.cwd = os.path.dirname(os.path.realpath(__file__))
        self.ini_file = os.path.join(self.cwd, '.', 'data', 'foobar.ini')
        self.invalid_ini_file = os.path.join(self.cwd, '.', 'data',
                                             'foobar-invalid.ini')

    def test_ini(self):
        provider = IniProviderBackend(self.ini_file)
        eq_('1', provider.get('Foo.Bar', None))
        eq_("Config Data", provider.get('Bar.Foo', None))
        eq_(None, provider.get('Bar', None))
        provider.delete('Foo.Bar')
        eq_(None, provider.get('Foo.Bar', None))
        provider.set('Foo.Bar', '2')
        eq_('2', provider.get('Foo.Bar', None))

        # New
        provider.set('FoosBallSection.Price', '$.50')
        eq_('$.50', provider.get('FoosBallSection.Price', None))

    @raises(ConfigException)
    def test_ini_with_invalid_delete_should_raise_exception(self):
        provider = IniProviderBackend(self.ini_file)
        provider.delete('Foo')

    @raises(ConfigException)
    def test_invalid_ini_should_raise_exception(self):
        IniProviderBackend(self.invalid_ini_file)

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .backends import get_provider
from .config import Config
import os


class Locator(object):
    def __init__(self, env_key, config_name, local_dir, system_dir):
        self.env_key = env_key
        self.config_name = config_name
        self.local_dir = local_dir
        self.system_dir = system_dir

    @property
    def config_name(self):
        return self._config_name

    @config_name.setter
    def config_name(self, value):
        self._config_name = value

    @property
    def env_key(self):
        return self._env_key

    @env_key.setter
    def env_key(self, value):
        self._env_key = value

    @property
    def local_dir(self):
        return self._local_dir

    @local_dir.setter
    def local_dir(self, value):
        self._local_dir = value

    @property
    def system_dir(self):
        return self._system_dir

    @system_dir.setter
    def system_dir(self, value):
        self._system_dir = value

    def get_config(self):
        for config_path in self.config_paths:
            if os.path.exists(config_path):
                provider = get_provider(config_path)
                return Config(provider)

    def get_config_paths(self):
        config_paths = [os.path.join(path, self.config_name)
                        for path in self._get_search_paths() if path is not
                        None]
        config_paths.insert(0, self._get_config_from_env())
        return config_paths

    def _get_config_from_env(self):
        return os.environ.get(self.env_key, None)

    def _get_search_paths(self):
        return [
            self._get_local_dir(),
            self._get_home_dir(),
            self._get_system_dir()
        ]

    def _get_local_dir(self):
        return self.local_dir

    def _get_home_dir(self):
        return os.path.expanduser("~")

    def _get_system_dir(self):
        return self.system_dir

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .base import ConfigException
from .base import FileProviderBackend
try:
    import ConfigParser as parser
except ImportError:
    import configparser as parser


class IniProviderBackend(FileProviderBackend):
    def __init__(self, file_path, *args, **kwargs):
        super(IniProviderBackend, self).__init__(file_path, *args, **kwargs)
        try:
            self._config = ToDictParser()
            self._config.read(self.file_path)
        except parser.MissingSectionHeaderError as e:
            raise ConfigException(e)

    def get(self, key, default_value=None):
        try:
            section, option = key.split('.')
            return self._config.get(section, option)
        except:
            return default_value

    def set(self, key, value):
        try:
            section, option = key.split('.')
            if not self._config.has_section(section):
                self._config.add_section(section)
            self._config.set(section, option, value)
        except Exception as e:
            raise ConfigException(e)

    def delete(self, key):
        try:
            key_split = key.split('.')
            if len(key_split) == 2:
                section, option = key_split[0], key_split[1]
                self._config.remove_option(section, option)
            else:
                raise ConfigException("Must include a section header")
        except Exception as e:
            raise ConfigException(e)

    def to_dict(self):
        try:
            return self._config.as_dict()
        except Exception as e:
            raise ConfigException(e)


class ToDictParser(parser.ConfigParser):
    def as_dict(self):
        d = dict(self._sections)
        for k in d:
            d[k] = dict(self._defaults, **d[k])
            d[k].pop('__name__', None)
        return d

# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import eq_
from nose.tools import raises
import os
from py_configurator.backends.base import ConfigException
from py_configurator.backends.yaml import YamlProviderBackend
import tempfile
import unittest


class TestYamlProviderBackend(unittest.TestCase):

    def setUp(self):
        self.cwd = os.path.dirname(os.path.realpath(__file__))
        self.yml_file = os.path.join(self.cwd, '.', 'data', 'foobar.yml')
        self.temp_file = tempfile.TemporaryFile()

    def test_yml(self):
        provider = YamlProviderBackend(self.yml_file)
        eq_(1, provider.get('Foo.Bar', None))
        eq_("Config Data", provider.get('Bar.Foo', None))
        eq_(None, provider.get('Bar2', None))
        provider.delete('Foo.Bar')
        eq_(None, provider.get('Foo.Bar', None))
        provider.set('Foo.Bar', '2')
        eq_('2', provider.get('Foo.Bar', None))

        # New
        provider.set('Foo', 'Ball')
        eq_('Ball', provider.get('Foo', None))

    @raises(ConfigException)
    def test_yml_invalid_set_should_raise_exception(self):
        provider = YamlProviderBackend(self.yml_file)
        provider.set(None, None)

    @raises(ConfigException)
    def test_yml_invalid_delete_should_raise_exception(self):
        provider = YamlProviderBackend(self.yml_file)
        provider.delete(None)

    @raises(ConfigException)
    def test_invalid_yml_location_should_raise_exception(self):
        YamlProviderBackend('/foo/bar')


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import ok_
import os
from py_configurator.backends import get_provider
from py_configurator.backends.ini import IniProviderBackend
import unittest


class TestProviderBackend(unittest.TestCase):

    def setUp(self):
        self.cwd = os.path.dirname(os.path.realpath(__file__))
        self.ini_file = os.path.join(self.cwd, '.', 'data', 'foobar.ini')

    def test_get_provider(self):
        provider = get_provider(self.ini_file)
        ok_(type(provider) == IniProviderBackend)


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import eq_
from py_configurator.backends.dict import DictionaryProviderBackend
import unittest


class TestDictionaryProviderBackend(unittest.TestCase):

    def test_dict(self):
        dict = {'foo': 'bar', 'too': 'much'}
        provider = DictionaryProviderBackend(dict)
        provider.set('python', 'rules')
        eq_('bar', provider.get('foo', None))
        eq_(None, provider.get('foo2', None))
        eq_(1, provider.get('foo3', 1))
        eq_('rules', provider.get('python', None))
        provider.delete('python')
        eq_(-1, provider.get('python', -1))


# vim: filetype=python



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .base import BaseProviderBackend


class DictionaryProviderBackend(BaseProviderBackend):
    def __init__(self, dict, *args, **kwargs):
        super(DictionaryProviderBackend, self).__init__(*args, **kwargs)
        self._dict = dict or {}

    def get(self, key, default_value):
        try:
            return self._dict[key]
        except:
            return default_value

    def set(self, key, value):
        self._dict[key] = value

    def delete(self, key):
        if key in self._dict:
            del self._dict[key]

    def to_dict(self):
        return self._dict

# vim: filetype=python
